# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/fccoelho/Documents/Projects_software/epigrass/epigrassqt4/Epigrass/neteditor.ui'
#
# Created: Tue Aug 11 14:10:16 2009
#      by: PyQt4 UI code generator 4.4.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(759, 588)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/egicon.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        MainWindow.setWindowIcon(icon)
        self.centralWidget = QtGui.QWidget(MainWindow)
        self.centralWidget.setObjectName("centralWidget")
        self.verticalLayout = QtGui.QVBoxLayout(self.centralWidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.splitter = QtGui.QSplitter(self.centralWidget)
        self.splitter.setOrientation(QtCore.Qt.Vertical)
        self.splitter.setObjectName("splitter")
        self.graphicsView = QtGui.QGraphicsView(self.splitter)
        self.graphicsView.setObjectName("graphicsView")
        self.dataPanel = QtGui.QTabWidget(self.splitter)
        self.dataPanel.setObjectName("dataPanel")
        self.Nodes = QtGui.QWidget()
        self.Nodes.setObjectName("Nodes")
        self.verticalLayout_2 = QtGui.QVBoxLayout(self.Nodes)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.nodeTable = QtGui.QTableWidget(self.Nodes)
        self.nodeTable.setAlternatingRowColors(True)
        self.nodeTable.setObjectName("nodeTable")
        self.nodeTable.setColumnCount(0)
        self.nodeTable.setRowCount(0)
        self.verticalLayout_2.addWidget(self.nodeTable)
        self.dataPanel.addTab(self.Nodes, "")
        self.Edges = QtGui.QWidget()
        self.Edges.setObjectName("Edges")
        self.verticalLayout_3 = QtGui.QVBoxLayout(self.Edges)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.edgeTable = QtGui.QTableWidget(self.Edges)
        self.edgeTable.setAlternatingRowColors(True)
        self.edgeTable.setObjectName("edgeTable")
        self.edgeTable.setColumnCount(0)
        self.edgeTable.setRowCount(0)
        self.verticalLayout_3.addWidget(self.edgeTable)
        self.dataPanel.addTab(self.Edges, "")
        self.verticalLayout.addWidget(self.splitter)
        MainWindow.setCentralWidget(self.centralWidget)
        self.menuBar = QtGui.QMenuBar(MainWindow)
        self.menuBar.setGeometry(QtCore.QRect(0, 0, 759, 25))
        self.menuBar.setObjectName("menuBar")
        self.menuFile = QtGui.QMenu(self.menuBar)
        self.menuFile.setObjectName("menuFile")
        self.menu_Export = QtGui.QMenu(self.menuFile)
        self.menu_Export.setObjectName("menu_Export")
        self.menu_View = QtGui.QMenu(self.menuBar)
        self.menu_View.setObjectName("menu_View")
        self.menuAnalyze = QtGui.QMenu(self.menuBar)
        self.menuAnalyze.setObjectName("menuAnalyze")
        self.menu_Debug = QtGui.QMenu(self.menuAnalyze)
        self.menu_Debug.setObjectName("menu_Debug")
        MainWindow.setMenuBar(self.menuBar)
        self.statusBar = QtGui.QStatusBar(MainWindow)
        self.statusBar.setObjectName("statusBar")
        MainWindow.setStatusBar(self.statusBar)
        self.toolBar = QtGui.QToolBar(MainWindow)
        self.toolBar.setObjectName("toolBar")
        MainWindow.addToolBar(QtCore.Qt.TopToolBarArea, self.toolBar)
        self.actionOpen_edges_file = QtGui.QAction(MainWindow)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/neteditor/openedges.png"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.actionOpen_edges_file.setIcon(icon1)
        self.actionOpen_edges_file.setObjectName("actionOpen_edges_file")
        self.actionOpen_Shapefile = QtGui.QAction(MainWindow)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/neteditor/openshape.png"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.actionOpen_Shapefile.setIcon(icon2)
        self.actionOpen_Shapefile.setObjectName("actionOpen_Shapefile")
        self.action_Save = QtGui.QAction(MainWindow)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/neteditor/document-save.png"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.action_Save.setIcon(icon3)
        self.action_Save.setObjectName("action_Save")
        self.actionSave_as = QtGui.QAction(MainWindow)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/neteditor/document-save-as.png"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.actionSave_as.setIcon(icon4)
        self.actionSave_as.setObjectName("actionSave_as")
        self.action_Graph = QtGui.QAction(MainWindow)
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/neteditor/graph.png"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.action_Graph.setIcon(icon5)
        self.action_Graph.setObjectName("action_Graph")
        self.actionE_xit = QtGui.QAction(MainWindow)
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(":/neteditor/application-exit.png"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.actionE_xit.setIcon(icon6)
        self.actionE_xit.setObjectName("actionE_xit")
        self.actionStats = QtGui.QAction(MainWindow)
        self.actionStats.setObjectName("actionStats")
        self.action_Close = QtGui.QAction(MainWindow)
        self.action_Close.setObjectName("action_Close")
        self.actionCheck_Graph = QtGui.QAction(MainWindow)
        self.actionCheck_Graph.setObjectName("actionCheck_Graph")
        self.actionDot = QtGui.QAction(MainWindow)
        self.actionDot.setObjectName("actionDot")
        self.actionGML = QtGui.QAction(MainWindow)
        self.actionGML.setObjectName("actionGML")
        self.actionPajek = QtGui.QAction(MainWindow)
        self.actionPajek.setObjectName("actionPajek")
        self.actionYAML = QtGui.QAction(MainWindow)
        self.actionYAML.setObjectName("actionYAML")
        self.menu_Export.addAction(self.actionDot)
        self.menu_Export.addAction(self.actionGML)
        self.menu_Export.addAction(self.actionPajek)
        self.menu_Export.addAction(self.actionYAML)
        self.menuFile.addAction(self.actionOpen_edges_file)
        self.menuFile.addAction(self.actionOpen_Shapefile)
        self.menuFile.addAction(self.action_Close)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.action_Save)
        self.menuFile.addAction(self.actionSave_as)
        self.menuFile.addAction(self.menu_Export.menuAction())
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionE_xit)
        self.menu_View.addAction(self.action_Graph)
        self.menu_Debug.addAction(self.actionCheck_Graph)
        self.menuAnalyze.addAction(self.menu_Debug.menuAction())
        self.menuAnalyze.addSeparator()
        self.menuAnalyze.addAction(self.actionStats)
        self.menuBar.addAction(self.menuFile.menuAction())
        self.menuBar.addAction(self.menu_View.menuAction())
        self.menuBar.addAction(self.menuAnalyze.menuAction())
        self.toolBar.addAction(self.actionOpen_edges_file)
        self.toolBar.addAction(self.actionOpen_Shapefile)
        self.toolBar.addAction(self.action_Save)
        self.toolBar.addSeparator()
        self.toolBar.addAction(self.action_Graph)
        self.toolBar.addSeparator()
        self.toolBar.addAction(self.actionE_xit)

        self.retranslateUi(MainWindow)
        self.dataPanel.setCurrentIndex(1)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QtGui.QApplication.translate("MainWindow", "Network Editor", None, QtGui.QApplication.UnicodeUTF8))
        self.graphicsView.setStatusTip(QtGui.QApplication.translate("MainWindow", "Map visualization window", None, QtGui.QApplication.UnicodeUTF8))
        self.Nodes.setToolTip(QtGui.QApplication.translate("MainWindow", "Node set", None, QtGui.QApplication.UnicodeUTF8))
        self.nodeTable.setStatusTip(QtGui.QApplication.translate("MainWindow", "Node list.", None, QtGui.QApplication.UnicodeUTF8))
        self.nodeTable.setSortingEnabled(False)
        self.dataPanel.setTabText(self.dataPanel.indexOf(self.Nodes), QtGui.QApplication.translate("MainWindow", "Nodes", None, QtGui.QApplication.UnicodeUTF8))
        self.edgeTable.setStatusTip(QtGui.QApplication.translate("MainWindow", "List of edges", None, QtGui.QApplication.UnicodeUTF8))
        self.edgeTable.setSortingEnabled(False)
        self.dataPanel.setTabText(self.dataPanel.indexOf(self.Edges), QtGui.QApplication.translate("MainWindow", "Edges", None, QtGui.QApplication.UnicodeUTF8))
        self.menuFile.setTitle(QtGui.QApplication.translate("MainWindow", "&File", None, QtGui.QApplication.UnicodeUTF8))
        self.menu_Export.setTitle(QtGui.QApplication.translate("MainWindow", "&Export", None, QtGui.QApplication.UnicodeUTF8))
        self.menu_View.setTitle(QtGui.QApplication.translate("MainWindow", "&View", None, QtGui.QApplication.UnicodeUTF8))
        self.menuAnalyze.setTitle(QtGui.QApplication.translate("MainWindow", "&Analyze", None, QtGui.QApplication.UnicodeUTF8))
        self.menu_Debug.setTitle(QtGui.QApplication.translate("MainWindow", "&Debug", None, QtGui.QApplication.UnicodeUTF8))
        self.toolBar.setWindowTitle(QtGui.QApplication.translate("MainWindow", "toolBar", None, QtGui.QApplication.UnicodeUTF8))
        self.actionOpen_edges_file.setText(QtGui.QApplication.translate("MainWindow", "Open &edges file", None, QtGui.QApplication.UnicodeUTF8))
        self.actionOpen_Shapefile.setText(QtGui.QApplication.translate("MainWindow", "Open &Shapefile", None, QtGui.QApplication.UnicodeUTF8))
        self.action_Save.setText(QtGui.QApplication.translate("MainWindow", "&Save", None, QtGui.QApplication.UnicodeUTF8))
        self.actionSave_as.setText(QtGui.QApplication.translate("MainWindow", "Save &as", None, QtGui.QApplication.UnicodeUTF8))
        self.action_Graph.setText(QtGui.QApplication.translate("MainWindow", "&Graph", None, QtGui.QApplication.UnicodeUTF8))
        self.actionE_xit.setText(QtGui.QApplication.translate("MainWindow", "E&xit", None, QtGui.QApplication.UnicodeUTF8))
        self.actionStats.setText(QtGui.QApplication.translate("MainWindow", "Stats", None, QtGui.QApplication.UnicodeUTF8))
        self.action_Close.setText(QtGui.QApplication.translate("MainWindow", "&Close", None, QtGui.QApplication.UnicodeUTF8))
        self.actionCheck_Graph.setText(QtGui.QApplication.translate("MainWindow", "Check Graph", None, QtGui.QApplication.UnicodeUTF8))
        self.actionDot.setText(QtGui.QApplication.translate("MainWindow", "Dot", None, QtGui.QApplication.UnicodeUTF8))
        self.actionGML.setText(QtGui.QApplication.translate("MainWindow", "GML", None, QtGui.QApplication.UnicodeUTF8))
        self.actionPajek.setText(QtGui.QApplication.translate("MainWindow", "Pajek", None, QtGui.QApplication.UnicodeUTF8))
        self.actionYAML.setText(QtGui.QApplication.translate("MainWindow", "YAML", None, QtGui.QApplication.UnicodeUTF8))

import epigrass_rc

if __name__ == "__main__":
    import sys
    app = QtGui.QApplication(sys.argv)
    MainWindow = QtGui.QMainWindow()
    ui = Ui_MainWindow()
    ui.setupUi(MainWindow)
    MainWindow.show()
    sys.exit(app.exec_())

